/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.relevance;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.relevance.vehicular.RelevanceCalculationComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSInformationConsumer.SiSConsumerHandle;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSRequest;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.SiSType;

public abstract class AbstractRelevanceCalculationComponent implements RelevanceCalculationComponent {

	private SiSComponent _sis;
	private Host _host;
	private List<SiSConsumerHandle> sisHandles = new ArrayList<>();

	public AbstractRelevanceCalculationComponent(Host pHost) {
		_host = pHost;
	}

	public abstract Map<SiSType<?>, SiSRequest> getRequiredInformation();

	@Override
	public Host getHost() {
		return _host;
	}

	protected SiSComponent getSiS() {
		if (_sis == null) {
			try {
				_sis = getHost().getComponent(SiSComponent.class);
			} catch (ComponentNotAvailableException e) {
				throw new AssertionError(e);
			}
		}
		return _sis;
	}

	@Override
	public void initialize() {
		Map<SiSType<?>, SiSRequest> metrics = getRequiredInformation();
		for (Map.Entry<SiSType<?>, SiSRequest> metric : metrics.entrySet()) {
			SiSConsumerHandle handle = getSiS().get().rawObservations(metric.getKey(), metric.getValue(), null);
			sisHandles.add(handle);
		}
	}
}
