/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.generator.lifetime;

import java.util.Random;

import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

public class GaussianLifetimeDistribution implements LifetimeDistribution {
	private Random eventDurationRandom = Randoms.getRandom(getClass());

	private static long _averageEventDuration;
	private static long _standardDeviationEventDuration;

	@XMLConfigurableConstructor({ "duration", "duration_variance" })
	public GaussianLifetimeDistribution(long pEventDuration, long pDeviation) {
		_averageEventDuration = pEventDuration;
		_standardDeviationEventDuration = pDeviation;
	}

	@Override
	public long getLifetime() {
		long delay = (long) ((eventDurationRandom.nextGaussian() * _standardDeviationEventDuration
				+ _averageEventDuration) / Time.SECOND) * Time.SECOND;
		return delay;
	}

	@Override
	public long getAverageLifetime() {
		return _averageEventDuration;
	}
}
