/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.api.EdgeController;

public class RoadNetwork implements RoadNetworkEdgeListener {
	public static RoadNetwork CURRENT_ROAD_NETWORK;

	private Map<String, RoadNetworkEdge> _roadNetwork;
	private boolean _changed = false;
	private EdgeController _edgeController;

	private List<RoadNetworkEdge> _visibleEdges = null;

	private Set<RoadNetworkEdge> _allEverActiveEdges = new HashSet<>();

	public RoadNetwork(SerializableRoadNetwork pRoadNetwork, EdgeController pEdgeController) {
		_roadNetwork = new HashMap<>();
		for (SerializableRoadNetworkEdge edge : pRoadNetwork.getAvailableEdges()) {
			RoadNetworkEdge roadNetworkEdge = new RoadNetworkEdge(edge, pEdgeController);
			roadNetworkEdge.registerEdgeListener(this);
			_roadNetwork.put(edge.getEdgeID(), roadNetworkEdge);
		}

		for (SerializableRoadNetworkEdge edge : pRoadNetwork.getAvailableEdges()) {
			RoadNetworkEdge roadNetworkEdge = _roadNetwork.get(edge.getEdgeID());

			for (String edgeID : edge.getAccessibleEdges()) {
				roadNetworkEdge.addConnectedEdge(_roadNetwork.get(edgeID));
			}
		}

		_edgeController = pEdgeController;
	}

	public RoadNetwork(Map<String, RoadNetworkEdge> pRoadNetwork, EdgeController pEdgeController) {
		_roadNetwork = pRoadNetwork;

		_edgeController = pEdgeController;
	}

	/**
	 * @param pRoadNetwork
	 * @param pB
	 */
	public RoadNetwork(Map<String, RoadNetworkEdge> pRoadNetwork, EdgeController pEdgeController, boolean pChanged) {
		this(pRoadNetwork, pEdgeController);
		_changed = true;
	}

	public List<RoadNetworkEdge> getAvailableEdges() {
		return new ArrayList<>(_roadNetwork.values());
	}

	public List<RoadNetworkEdge> getUsableEdges() {
		if (_visibleEdges == null) {
			if (!_edgeController.isObservedAreaSet()) {
				_visibleEdges = new ArrayList<>(_roadNetwork.values());
			} else {
				List<RoadNetworkEdge> allEdges = new ArrayList<>(_roadNetwork.values());
				List<RoadNetworkEdge> result = new ArrayList<>();
				
				for (RoadNetworkEdge roadNetworkEdge : allEdges) {
					if (_edgeController.isEdgeUsable(roadNetworkEdge.getEdgeID())) {
						result.add(roadNetworkEdge);
					}
				}
				_visibleEdges = result;
			}
		}
		
		return _visibleEdges;
	}

	public Set<RoadNetworkEdge> getAllEverActiveEdges() {
		return _allEverActiveEdges;
	}

	public RoadNetworkEdge getEdge(String pEdge) {
		return _roadNetwork.get(pEdge);
	}

	/**
	 * @return
	 */
	public boolean hasChanged() {
		return _changed;
	}

	@Override
	public void edgeActivated(RoadNetworkEdge pEdge, RoadProperty pProperty) {
		_allEverActiveEdges.add(pEdge);
	}

	@Override
	public void edgeDeactivated(RoadNetworkEdge pEdge, RoadProperty pProperty) {

	}
}
