package de.tudarmstadt.maki.simonstrator.api.component.vehicular.subscriptions.topic;

import java.util.regex.Pattern;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.component.pubsub.PubSubComponent;
import de.tudarmstadt.maki.simonstrator.api.component.pubsub.attribute.Topic;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.costs.PropertyBenefitEstimator;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.costs.PropertyBenefitEstimatorFactory;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.costs.VehicularPropertyCostEstimator;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.RoadInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.subscriptions.SubscriptionTopicType;

public class CostBasedTopic implements ProcessedTopic, Comparable<CostBasedTopic> {

	public static final String COST_TOPIC_START = "MinCosts(";
	public static final String COST_TOPIC_END = ")";
	public static final Pattern COST_TOPIC_PATTERN = Pattern
			.compile(COST_TOPIC_START.replace("(", "\\(") + ".*" + COST_TOPIC_END.replace(")", "\\)"));

	private Topic _topic;
	private double _costs;

	public CostBasedTopic(PubSubComponent pPubSub, double pCosts) {
		_topic = pPubSub.createTopic(COST_TOPIC_START + Double.toString(pCosts) + COST_TOPIC_END);
		_costs = pCosts;
	}

	@Override
	public Topic getTopic() {
		return _topic;
	}

	public double getCosts() {
		return _costs;
	}

	@Override
	public SubscriptionTopicType getTopicType() {
		return SubscriptionTopicType.COST_BASED;
	}

	@Override
	public boolean matchesInformationRequirements(PointInformation<?> pInformation) {
		if (pInformation instanceof RoadInformation) {
			RoadInformation roadInfo = (RoadInformation) pInformation;
			PropertyBenefitEstimator costEstimator = PropertyBenefitEstimatorFactory.getPropertyBenefitEstimator();
			double costsForMissingInformation = ((VehicularPropertyCostEstimator) costEstimator)
					.calculateCostsIfUnknown(roadInfo.getValue());

			if (costsForMissingInformation >= _costs) {
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean includes(ProcessedTopic pTopic) {
		if (pTopic instanceof CostBasedTopic) {
			return _costs <= ((CostBasedTopic) pTopic)._costs;
		}
		return false;
	}

	@Override
	public boolean matchesHost(Host pHost) {
		return true;
	}

	@Override
	public int compareTo(CostBasedTopic pArg0) {
		return Double.compare(_costs, pArg0._costs);
	}

	@Override
	public String toString() {
		return "Costs: " + Double.toString(_costs);
	}

}
